/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QDockWidget>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QToolButton>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QCheckBox>

#include "machine_dock.h"


TMachineDock::TMachineDock(
	const QWidget						*parent)
:QObject(const_cast<QWidget*>(parent))
{
	QFrame								*upper_separator_hline;
	QFrame								*lower_separator_hline;
	QGridLayout							*grid_layout;
	QHBoxLayout							*button_protocol_layout;
	QSpacerItem							*button_protocol_hspacer;
	QHBoxLayout							*button_move_jogbox_layout;
	QSpacerItem							*button_move_jogbox_hspacer;
	QToolButton							*jogbox_toggle_button;
	QToolButton							*move_button;
	QLabel								*max_x_label;
	QLabel								*max_y_label;
	QLabel								*max_z_label;
	QLabel								*min_x_label;
	QLabel								*min_y_label;
	QLabel								*min_z_label;
	QLabel								*move_x_label;
	QLabel								*move_y_label;
	QLabel								*move_z_label;
	QLabel								*path_buffer_label;
	QSpacerItem							*machine_vspacer;
	
	d_machine_dock = new QDockWidget(const_cast<QWidget*>(parent));
	
	d_machine_widget = new QWidget();
	d_machine_dock->setWidget(d_machine_widget);
	
	grid_layout = new QGridLayout(d_machine_widget);
	grid_layout->setSpacing(4);

	button_protocol_layout = new QHBoxLayout();
	
	d_protocol_button = new QToolButton(d_machine_widget);
	button_protocol_layout->addWidget(d_protocol_button);
	
	d_protocol_label = new QLabel(d_machine_widget);
	button_protocol_layout->addWidget(d_protocol_label);

	d_serial_connect_button = new QToolButton(d_machine_widget);
	button_protocol_layout->addWidget(d_serial_connect_button);
	
	button_protocol_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_protocol_layout->addItem(button_protocol_hspacer);
	
	grid_layout->addLayout(button_protocol_layout,0,0,1,2);

	upper_separator_hline = new QFrame(d_machine_widget);
	upper_separator_hline->setFrameShape(QFrame::HLine);
	upper_separator_hline->setFrameShadow(QFrame::Sunken);
	grid_layout->addWidget(upper_separator_hline,1,0,1,2);
	
	min_x_label = new QLabel(d_machine_widget);
	min_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(min_x_label,2,0,1,1);
	
	d_min_x_edit = new QLineEdit(d_machine_widget);
	d_min_x_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_min_x_edit,2,1,1,1);

	max_x_label = new QLabel(d_machine_widget);
	max_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(max_x_label,3,0,1,1);
	
	d_max_x_edit = new QLineEdit(d_machine_widget);
	d_max_x_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_max_x_edit,3,1,1,1);

	min_y_label = new QLabel(d_machine_widget);
	min_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(min_y_label,4,0,1,1);

	d_min_y_edit = new QLineEdit(d_machine_widget);
	d_min_y_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_min_y_edit,4,1,1,1);
	
	max_y_label = new QLabel(d_machine_widget);
	max_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(max_y_label,5,0,1,1);
	
	d_max_y_edit = new QLineEdit(d_machine_widget);
	d_max_y_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_max_y_edit,5,1,1,1);
	
	min_z_label = new QLabel(d_machine_widget);
	min_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(min_z_label,6,0,1,1);
	
	d_min_z_edit = new QLineEdit(d_machine_widget);
	d_min_z_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_min_z_edit,6,1,1,1);

	max_z_label = new QLabel(d_machine_widget);
	max_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(max_z_label,7,0,1,1);
	
	d_max_z_edit = new QLineEdit(d_machine_widget);
	d_max_z_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_max_z_edit,7,1,1,1);
	
	d_override_tip_check = new QCheckBox(d_machine_widget);
	grid_layout->addWidget(d_override_tip_check,8,0,1,1);

	d_override_tip_spin = new QDoubleSpinBox(d_machine_widget);
	d_override_tip_spin->setAlignment(Qt::AlignCenter);
	d_override_tip_spin->setRange(0.1,8.0);
	d_override_tip_spin->setDecimals(4);
	grid_layout->addWidget(d_override_tip_spin,8,1,1,1);
	
	d_generate_noise_check = new QCheckBox(d_machine_widget);
	grid_layout->addWidget(d_generate_noise_check,9,0,1,1);
	
	d_generate_noise_spin = new QDoubleSpinBox(d_machine_widget);
	d_generate_noise_spin->setAlignment(Qt::AlignCenter);
	d_generate_noise_spin->setRange(0.001,0.25);
	d_generate_noise_spin->setDecimals(4);
	grid_layout->addWidget(d_generate_noise_spin,9,1,1,1);

	path_buffer_label = new QLabel(d_machine_widget);
	path_buffer_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(path_buffer_label,10,0,1,1);
	
	d_touch_buffer_spin = new QSpinBox(d_machine_widget);
	d_touch_buffer_spin->setAlignment(Qt::AlignCenter);
	d_touch_buffer_spin->setRange(0,50);
	grid_layout->addWidget(d_touch_buffer_spin,10,1,1,1);
	
	lower_separator_hline = new QFrame(d_machine_widget);
	lower_separator_hline->setFrameShape(QFrame::HLine);
	lower_separator_hline->setFrameShadow(QFrame::Sunken);
	grid_layout->addWidget(lower_separator_hline,11,0,1,2);

	button_move_jogbox_layout = new QHBoxLayout();
	
	move_button = new QToolButton(d_machine_widget);
	button_move_jogbox_layout->addWidget(move_button);

	button_move_jogbox_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_move_jogbox_layout->addItem(button_move_jogbox_hspacer);
	
	jogbox_toggle_button = new QToolButton(d_machine_widget);
	button_move_jogbox_layout->addWidget(jogbox_toggle_button);

	grid_layout->addLayout(button_move_jogbox_layout,12,0,1,2);
	
	move_x_label = new QLabel(d_machine_widget);
	move_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(move_x_label,13,0,1,1);
	
	d_move_x_edit = new QLineEdit(d_machine_widget);
	d_move_x_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_move_x_edit,13,1,1,1);
	
	move_y_label = new QLabel(d_machine_widget);
	move_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(move_y_label,14,0,1,1);
	
	d_move_y_edit = new QLineEdit(d_machine_widget);
	d_move_y_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_move_y_edit,14,1,1,1);
	
	move_z_label = new QLabel(d_machine_widget);
	move_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	grid_layout->addWidget(move_z_label,15,0,1,1);
	
	d_move_z_edit = new QLineEdit(d_machine_widget);
	d_move_z_edit->setAlignment(Qt::AlignCenter);
	grid_layout->addWidget(d_move_z_edit,15,1,1,1);

	machine_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	grid_layout->addItem(machine_vspacer,16,0,1,2);
	
	QWidget::setTabOrder(d_protocol_button,d_serial_connect_button);
	QWidget::setTabOrder(d_serial_connect_button,d_min_x_edit);
	QWidget::setTabOrder(d_min_x_edit,d_max_x_edit);
	QWidget::setTabOrder(d_max_x_edit,d_min_y_edit);
	QWidget::setTabOrder(d_min_y_edit,d_max_y_edit);
	QWidget::setTabOrder(d_max_y_edit,d_min_z_edit);
	QWidget::setTabOrder(d_min_z_edit,d_max_z_edit);
	QWidget::setTabOrder(d_max_z_edit,d_override_tip_spin);
	QWidget::setTabOrder(d_override_tip_spin,d_generate_noise_spin);
	QWidget::setTabOrder(d_generate_noise_spin,d_touch_buffer_spin);
	QWidget::setTabOrder(d_touch_buffer_spin,move_button);
	QWidget::setTabOrder(move_button,jogbox_toggle_button);
	QWidget::setTabOrder(jogbox_toggle_button,d_move_x_edit);
	QWidget::setTabOrder(d_move_x_edit,d_move_y_edit);
	QWidget::setTabOrder(d_move_y_edit,d_move_z_edit);
	
	// defaults
	d_touch_buffer_spin->setValue(0);
	d_override_tip_spin->setEnabled(false);
	d_generate_noise_spin->setEnabled(false);
	
	d_machine_dock->setWindowTitle(QStringLiteral("Machine"));
	
	move_button->setText(QStringLiteral("Move"));
	jogbox_toggle_button->setText(QStringLiteral("Jogbox"));
	min_x_label->setText(QStringLiteral("Min X:"));
	max_x_label->setText(QStringLiteral("Max X:"));
	min_y_label->setText(QStringLiteral("Min Y:"));
	max_y_label->setText(QStringLiteral("Max Y:"));
	min_z_label->setText(QStringLiteral("Min Z:"));
	max_z_label->setText(QStringLiteral("Max Z:"));
	path_buffer_label->setText(QStringLiteral("Path Buffer:"));
	move_x_label->setText(QStringLiteral("Move X:"));
	move_y_label->setText(QStringLiteral("Move Y:"));
	move_z_label->setText(QStringLiteral("Move Z:"));
	d_protocol_button->setText(QStringLiteral("Protocol"));
	d_protocol_label->setText(QStringLiteral("Serial"));
	d_serial_connect_button->setText(QStringLiteral("Open COMx"));
	d_override_tip_check->setText(QStringLiteral("Override Tip:"));
	d_generate_noise_check->setText(QStringLiteral("Touch Noise:"));

	connect(d_min_x_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
	connect(d_min_y_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
	connect(d_min_z_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
	connect(d_max_x_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
	connect(d_max_y_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
	connect(d_max_z_edit,&QLineEdit::editingFinished,this,&TMachineDock::Volume_Changed);
		
	connect(jogbox_toggle_button,&QToolButton::clicked,this,&TMachineDock::Show_Hide_Jogbox);
	connect(move_button,&QToolButton::clicked,this,&TMachineDock::Move_Clicked);
	connect(d_protocol_button,&QToolButton::clicked,this,&TMachineDock::Change_Protocol);
	connect(d_serial_connect_button,&QToolButton::clicked,this,&TMachineDock::Connect_Serial);
	connect(d_override_tip_check,&QCheckBox::toggled,d_override_tip_spin,&QDoubleSpinBox::setEnabled);
	connect(d_generate_noise_check,&QCheckBox::toggled,d_generate_noise_spin,&QDoubleSpinBox::setEnabled);
	connect(d_override_tip_check,&QCheckBox::toggled,this,&TMachineDock::Override_Tip_State_Changed);
	connect(d_generate_noise_check,&QCheckBox::toggled,this,&TMachineDock::Generate_Noise_State_Changed);
	connect(d_override_tip_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TMachineDock::Override_Tip_Diameter_Changed);
	connect(d_generate_noise_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TMachineDock::Generate_Noise_Value_Changed);
	
	connect(d_touch_buffer_spin,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),this,&TMachineDock::Touch_Buffer_Changed);
}

TMachineDock::~TMachineDock(void)
{
}

TVector3 TMachineDock::Min_Machine(void) const
{
	TVector3							pt_min;
	TVector3							pt_max;
	double								dval;
	
	pt_min.x = d_min_x_edit->text().toDouble();
	pt_min.y = d_min_y_edit->text().toDouble();
	pt_min.z = d_min_z_edit->text().toDouble();
	
	pt_max.x = d_max_x_edit->text().toDouble();
	pt_max.y = d_max_y_edit->text().toDouble();
	pt_max.z = d_max_z_edit->text().toDouble();
	
	if(pt_min.x > pt_max.x)
	{
		dval = pt_min.x;
		pt_min.x = pt_max.x;
		pt_max.x = dval;
	}
	
	if(pt_min.y > pt_max.y)
	{
		dval = pt_min.y;
		pt_min.y = pt_max.y;
		pt_max.y = dval;
	}
	
	if(pt_min.z > pt_max.z)
	{
		dval = pt_min.z;
		pt_min.z = pt_max.z;
		pt_max.z = dval;
	}

	
	return pt_min;
}

TVector3 TMachineDock::Max_Machine(void) const
{
	TVector3							pt_min;
	TVector3							pt_max;
	double								dval;
	
	pt_min.x = d_min_x_edit->text().toDouble();
	pt_min.y = d_min_y_edit->text().toDouble();
	pt_min.z = d_min_z_edit->text().toDouble();
	
	pt_max.x = d_max_x_edit->text().toDouble();
	pt_max.y = d_max_y_edit->text().toDouble();
	pt_max.z = d_max_z_edit->text().toDouble();
	
	if(pt_min.x > pt_max.x)
	{
		dval = pt_min.x;
		pt_min.x = pt_max.x;
		pt_max.x = dval;
	}
	
	if(pt_min.y > pt_max.y)
	{
		dval = pt_min.y;
		pt_min.y = pt_max.y;
		pt_max.y = dval;
	}
	
	if(pt_min.z > pt_max.z)
	{
		dval = pt_min.z;
		pt_min.z = pt_max.z;
		pt_max.z = dval;
	}
	
	
	return pt_max;
}

bool TMachineDock::Override_Tip_Enabled(void) const
{
	return d_override_tip_check->isChecked();
}

double TMachineDock::Tip_Diameter(void) const
{
	return d_override_tip_spin->value();
}

bool TMachineDock::Generate_Noise_Enabled(void) const
{
	return d_generate_noise_check->isChecked();
}

double TMachineDock::Noise_Value(void) const
{
	return d_generate_noise_spin->value();
}

TVector3 TMachineDock::Move_Position(void) const
{
	TVector3							pt;
	
	pt.x = d_move_x_edit->text().toDouble();
	pt.y = d_move_y_edit->text().toDouble();
	pt.z = d_move_z_edit->text().toDouble();
	
	return pt;
}

int TMachineDock::Touch_Buffer_Size(void) const
{
	return d_touch_buffer_spin->value();
}

void TMachineDock::Set_Min_Machine(
	const TVector3 						&pnt)
{
	d_min_x_edit->setText(QString("%1").arg(pnt.x,0,'f',3));
	d_min_y_edit->setText(QString("%1").arg(pnt.y,0,'f',3));
	d_min_z_edit->setText(QString("%1").arg(pnt.z,0,'f',3));
}

void TMachineDock::Set_Max_Machine(
	const TVector3 						&pnt)
{
	d_max_x_edit->setText(QString("%1").arg(pnt.x,0,'f',3));
	d_max_y_edit->setText(QString("%1").arg(pnt.y,0,'f',3));
	d_max_z_edit->setText(QString("%1").arg(pnt.z,0,'f',3));
}

void TMachineDock::Set_Tip_Diameter(
	const double						&value)
{
	d_override_tip_spin->setValue(value);
}

void TMachineDock::Set_Noise(
	const double						&value)
{
	d_generate_noise_spin->setValue(value);
}

void TMachineDock::Enable_Override_Tip(
	const bool 							state)
{
	d_override_tip_check->setChecked(state);
	d_override_tip_spin->setEnabled(state);
}

void TMachineDock::Enable_Noise(
	const bool 							state)
{
	d_generate_noise_check->setChecked(state);
	d_generate_noise_spin->setEnabled(state);
}

void TMachineDock::Set_Move_Position(
	const TVector3						&pnt)
{
	d_move_x_edit->setText(QString("%1").arg(pnt.x,0,'f',3));
	d_move_y_edit->setText(QString("%1").arg(pnt.y,0,'f',3));
	d_move_z_edit->setText(QString("%1").arg(pnt.z,0,'f',3));
}

void TMachineDock::Set_Touch_Buffer_Size(
	const int							buffer_size)
{
	d_touch_buffer_spin->setValue(buffer_size);
}

void TMachineDock::Enable_Protocol(
	const bool							state)
{
	d_protocol_button->setEnabled(state);
}

void TMachineDock::Set_Serial_Connect_Button(
	const bool							enabled,
	const QString						&text)
{
	if(enabled)
	{
		d_serial_connect_button->setEnabled(true);
		d_protocol_label->setText(QStringLiteral("Serial"));
	}
	else
	{
		d_serial_connect_button->setEnabled(false);
		d_protocol_label->setText(QStringLiteral("Ethernet"));
	}
	
	if(text.length())
	{
		d_serial_connect_button->setText(text);
	}
}

void TMachineDock::Move_Clicked(void)
{
	double								x,y,z;
	bool								state;
	
	x = d_move_x_edit->text().toDouble(&state);
	if(!state) return;
	
	y = d_move_y_edit->text().toDouble(&state);
	if(!state) return;
	
	z = d_move_z_edit->text().toDouble(&state);
	if(!state) return;
	
	emit Move(x,y,z);
}
